﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MyNoteManager.Interfaces;
using DHI.Solutions.Generic.Business;
using System.ComponentModel;

namespace MyNoteManager.Business
{
    public class MyNote : BaseGroupedEntity, IMyNote
    {
        private string _name;
        private string _text;

        /// <summary>
        /// Initializes a new instance of the Indicator class.
        /// </summary>
        protected internal MyNote()
        {
        }
        
        /// <summary>
        /// Gets or sets the Name of MyNote
        /// </summary>
        public virtual string Name
        {
            get
            {
                return _name;
            }

            set
            {
                if (_name != value)
                {
                    OnPropertyChanging("Name");
                    _name = value;
                    SetModified(true);
                    OnPropertyChanged("Name");
                }
            }
        }

        /// <summary>
        /// Gets or sets the Text of MyNote
        /// </summary>
        public virtual string Text
        {
            get
            {
                return _text;
            }

            set
            {
                if (_name != value)
                {
                    OnPropertyChanging("Text");
                    _text = value;
                    SetModified(true);
                    OnPropertyChanged("Text");
                }
            }
        }

        /// <inheritdoc />
        public virtual object Clone()
        {
            MyNote note = new MyNote();
            note.Name = this.Name;
            note.Text = this.Text;
            note.GroupId = this.GroupId;

            return note;
        }

        /// <inheritdoc />
        public virtual bool Validate(out string error)
        {
            bool valid = _ValidateName(Name, out error);

            if (!valid)
            {
                return false;
            }

            return true;
        }

        private bool _ValidateName(string name, out string error)
        {
            error = null;

            if (string.IsNullOrEmpty(name) || name.Trim() == string.Empty)
            {
                error = Resources.BaseNamedEntity_Name_CantBeEmpty;
            }
            else if (name.Length > 255)
            {
                error = Resources.BaseNamedEntity_Name_Max_255_Characters;
            }
            else if (name.Contains('/'))
            {
                error = Resources.BaseNamedGroup_Name_CantContainSlash;
            }

            return error == null;
        }


        /// <summary>
        /// Gets or sets indicator group parent.
        /// </summary>
        /// <remarks>
        /// Don't use this property for accessing Parent data. Only Parent.Id property is guaranteed to be set correctly.
        /// </remarks>
        [Browsable(false)]
        protected internal virtual MyNoteGroup Group
        {
            get
            {
                if (GroupId == null)
                {
                    return null;
                }
                else
                {
                    return new MyNoteGroup() { Id = GroupId };
                }
            }

            set
            {
                if (value == null)
                {
                    GroupId = null;
                }
                else
                {
                    GroupId = value.Id;
                }
            }
        }
    }
}
