﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using DHI.Solutions.Generic;
using MyNoteManager.Interfaces;
using DHI.Solutions.Generic.Data;

namespace MyNoteManager.UI
{
        public partial class MyNoteDataView : Form, IDataViewControl
    {
        private IShell _shell;
        private IMyNoteModule _myNoteModule;
        private IMyNote _myNote;
        private IMyNote _originalMyNote;
        private bool _initialized;

        public MyNoteDataView()
        {
            InitializeComponent();
        }

        public bool CanClose
        {
            get
            {
                return true;
            }
        }

        public DHI.Solutions.Generic.DialogResult DataViewResult
        {
            get
            {
                return DHI.Solutions.Generic.DialogResult.None;
            }
        }

        public ToolStripContainer ToolStripContainer
        {
            get { return _toolStripContainerMain; }
        }

        public string Caption
        {
            get
            {
                return this.Text;
            }

            set
            {
                if (this.Parent is Crownwood.DotNetMagic.Controls.TabControl)
                {
                    (this.Parent as Crownwood.DotNetMagic.Controls.TabControl).SelectedTab.Title = value;
                    this.Text = value;
                }
            }
        }

        public  Control Control
        {
            get { return this; }
        }

        public int IconHandle
        {
            get
            {
                return this.Icon.Handle.ToInt32();
            }
        }

        public void Initialize(IShell shell)
        {
            if (_initialized)
            {
                return;
            }

            try
            {
                _shell = shell;
                _shell.DataViewControls.ControlRemoving += new EventHandler<CancelControlEventArgs>(_DataViewControls_ControlRemoving);
                _myNoteModule = (IMyNoteModule)_shell.Application.Modules.Get(typeof(IMyNoteModule));

                _initialized = true;
            }
            catch (Exception ex)
            {
                bool rethrow;

                rethrow = DssEntlib.HandleUILayerException(ex);
                if (rethrow)
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Gets or sets an myNote which this forrm represents.
        /// </summary>
        public IMyNote MyNote
        {
            get
            {
                return _myNote;
            }

            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("MyNote can't be null.");
                }

                if (_myNote != null)
                {
                    throw new ArgumentNullException("MyNote is already set.");
                }

                try
                {
                    _myNote = value;
                    _originalMyNote = (IMyNote)_myNote.Clone();

                    _myNote.SetModified(false);
                    _myNote.PropertyChanged += new PropertyChangedEventHandler(_MyNote_PropertyChanged);

                    this.tbName.Text = _myNote.Name;
                    this.tbText.Text = _myNote.Text;
                    this.tbVersion.Text = (_myNote as IDTO).Version.ToString();

                    // form title
                    if (string.IsNullOrEmpty(_myNote.Name))
                    {
                        this.Text = Resources.MyNoteDataView_New;
                    }
                    else
                    {
                        this.Text = _myNote.Name;
                    }
                }
                catch (Exception ex)
                {
                    bool rethrow;

                    rethrow = DssEntlib.HandleUILayerException(ex);
                    if (rethrow)
                    {
                        throw;
                    }
                }
            }
        }
        
        public bool Save()
        {
            bool success = false;
            try
            {
                this.Validate();

                try
                {
                    if (_myNote.Id == null || !_myNoteModule.MyNoteList.Exists(_myNote.Id))
                    {
                        // add myNote
                        _myNoteModule.MyNoteList.Add(_myNote);
                        success = true;

                        this.Caption = _myNote.Name;
                    }
                    else
                    {
                        _myNoteModule.MyNoteList.Update(_myNote);
                        success = true;
                        if (this.Caption[this.Caption.Length - 1] == '*')
                        {
                            this.Caption = this.Caption.Remove(this.Caption.Length - 1, 1);
                        }
                    }

                    this.tbVersion.Text = (_myNote as IDTO).Version.ToString();
                    _originalMyNote = (IMyNote)_myNote.Clone();
                }
                catch (EntityValidationException ex)
                {
                    // myNote can't be saved, restore original myNote 
                    _myNote.Name = _originalMyNote.Name;
                    _myNote.Text = _originalMyNote.Text;
                    _myNote.GroupId = _originalMyNote.GroupId;
                    MessageBox.Show(ex.Message, string.Empty, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            catch (Exception ex)
            {
                bool rethrow;

                rethrow = DssEntlib.HandleUILayerException(ex);
                if (rethrow)
                {
                    throw;
                }
            }

            return success;
        }

        private void _DataViewControls_ControlRemoving(object sender, CancelControlEventArgs e)
        {
            if (e.Control == this && _myNote.IsModified)
            {
                System.Windows.Forms.DialogResult res = MessageBox.Show(Resources.MyNoteDataView_DoYouWantToSaveChanges, string.Empty, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
                if (res == System.Windows.Forms.DialogResult.Yes)
                {
                    bool success = Save();
                    if (!success)
                    {
                        e.Cancel = true;
                    }
                }
                else if (res == System.Windows.Forms.DialogResult.Cancel)
                {
                    e.Cancel = true;
                }
            }
        }

        private void _MyNote_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (this.Caption[this.Caption.Length - 1] != '*')
            {
                this.Caption = this.Caption + '*';
            }
        }

        private void tbName_TextChanged(object sender, EventArgs e)
        {
            _myNote.Name = tbName.Text;
        }

        private void tbText_TextChanged(object sender, EventArgs e)
        {
            _myNote.Text = tbText.Text;
        }
    }
}
