﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using DHI.Solutions.Generic;
using DHI.Solutions.Generic.UI;
using MyNoteManager.Interfaces;

namespace MyNoteManager.UI
{
    public class MyNoteExplorer: TreeExplorer
    {
        private const string _branchIcon = "branch";
        private const string _groupIcon = "group";
        private const string _noteIcon = "note";

        private IMyNoteModule _module;

        /// <summary>
        /// Initializes a new instance of the MyNoteExplorer class.
        /// </summary>
        public MyNoteExplorer()
        {
            _InitializeComponent();
        }

        /// <inheritdoc />
        public override void Initialize(IShell shell)
        {
            base.Initialize(shell);

            _module = (IMyNoteModule)_shell.Application.Modules.Get(typeof(IMyNoteModule));

            ImageList imageList = new ImageList();
            imageList.Images.Add(_branchIcon, Resources.explorer_branch);
            imageList.Images.Add(_groupIcon, Resources.explorer_group);
            imageList.Images.Add(_noteIcon, Resources.explorer_note);
            this.Icons = imageList;

            GroupedListEntityTreeExplorerBranch<IMyNote, IMyNoteGroup> dbMyNotes = new GroupedListEntityTreeExplorerBranch<IMyNote, IMyNoteGroup>();
            dbMyNotes.Enabled = true;
            dbMyNotes.Visible = true;
            dbMyNotes.EntityList = _module.MyNoteList;
            dbMyNotes.GroupEntityList = _module.MyNoteGroupList;
            dbMyNotes.EntityName = Resources.MyNote_EntityName;
            dbMyNotes.BranchName = Resources.DbTreeBranch_RootName;

            MyNoteGroupExplorerPerspective groupPerspective = new MyNoteGroupExplorerPerspective(_shell, dbMyNotes);
            groupPerspective.Name = Resources.MyNoteGroupPerspective_Name;
            groupPerspective.ImageIndexes.Add(MyNoteGroupExplorerPerspective.ImageKeyEntity, imageList.Images.IndexOfKey(_noteIcon));
            groupPerspective.ImageIndexes.Add(MyNoteGroupExplorerPerspective.ImageKeyBranch, imageList.Images.IndexOfKey(_branchIcon));
            groupPerspective.ImageIndexes.Add(MyNoteGroupExplorerPerspective.ImageKeyGroup, imageList.Images.IndexOfKey(_groupIcon));

            dbMyNotes.AvailablePerspectives.Add(groupPerspective);
            dbMyNotes.CurrentPerspective = groupPerspective;

            this.AddBranch(dbMyNotes);
        }

        /// <inheritdoc />
        public override string Caption
        {
            get
            {
                return Resources.MyNoteExplorer_Caption;
            }
        }

        private void _InitializeComponent()
        {
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(MyNoteExplorer));
            this.SuspendLayout();

            // ScriptExplorer
            this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
            this.ClientSize = new System.Drawing.Size(243, 616);
            this.Icon = Resources.explorer;
            this.Name = "MyNoteExplorer";

            this.ResumeLayout(false);
        }
    }
}
