﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using DHI.Solutions.Generic;
using DHI.Solutions.Generic.UI;
using MyNoteManager.Interfaces;


namespace MyNoteManager.UI
{
    internal class MyNoteGroupExplorerPerspective : GroupedEntityExplorerPerspective<IMyNote, IMyNoteGroup>
    {
        #region Constants

        public static string ActionCodeEditMyNote
        {
            get
            {
                return "EDITMYNOTE";
            }
        }

        public static string ActionGroupMyNote
        {
            get
            {
                return "MYNOTE";
            }
        }

        #endregion

        private IMyNoteModule _module;
        private Dictionary<object, MyNoteProperties> _myNotePropertiesCache;

        public MyNoteGroupExplorerPerspective(IShell shell, GroupedListEntityTreeExplorerBranch<IMyNote, IMyNoteGroup> treeBranch)
            : base(shell, treeBranch)
        {
            _module = (IMyNoteModule)_shell.Application.Modules.Get(typeof(IMyNoteModule));
            _myNotePropertiesCache = new Dictionary<object, MyNoteProperties>();
        }

        protected override string ManagerName
        {
            get { return Resources.Module_Name; }
        }

        public override IList<IExplorerNodeAction> GetNodeActions(IList<ITreeNode> nodeList, IDataObject contextData)
        {
            // process inherited actions
            IList<IExplorerNodeAction> originalActions = base.GetNodeActions(nodeList, contextData);
            List<IExplorerNodeAction> actions = new List<IExplorerNodeAction>();

            foreach (IExplorerNodeAction action in originalActions)
            {
                // use only selected actions from base classes
                if (action.ActionCode == TreeExplorer.ActionCodeDrag ||
                    action.ActionCode == TreeExplorer.ActionCodeCut ||
                    action.ActionCode == TreeExplorer.ActionCodeCopy ||
                    action.ActionCode == TreeExplorer.ActionCodeCopyPath ||
                    action.ActionCode == TreeExplorer.ActionCodePasteCopy ||
                    action.ActionCode == TreeExplorer.ActionCodePasteMove ||
                    action.ActionCode == TreeExplorer.ActionCodeRefresh ||
                    action.ActionCode == TreeExplorer.ActionCodeDelete ||
                    action.ActionCode == TreeExplorer.ActionCodeAddGroup ||
                    action.ActionCode == TreeExplorer.ActionCodeRenameEntity ||
                    action.ActionCode == TreeExplorer.ActionCodeRenameGroup)
                {
                    actions.Add(action);
                }
            }

            int myNoteNodes = 0;
            int groupNodes = 0;
            int branchNodes = 0;

            foreach (ITreeNode node in nodeList)
            {
                if (node.Tag is IMyNote)
                {
                    myNoteNodes++;
                }
                else if (node.Tag is IMyNoteGroup)
                {
                    groupNodes++;
                }
                else if (node.Tag is IExplorerBranch)
                {
                    branchNodes++;
                }
            }

            // add myNote
            if (groupNodes > 0 || branchNodes > 0)
            {
                NodeAction addMyNoteAction = new NodeAction(nodeList);
                addMyNoteAction.Text = Resources.MyNoteGroupExplorerPerspective_AddNewMyNote;
                addMyNoteAction.Enabled = myNoteNodes == 0 && (groupNodes == 1 || branchNodes == 1);
                addMyNoteAction.ActionCode = TreeExplorer.ActionCodeAddEntity;
                addMyNoteAction.ActionGroup = ActionGroupNew;
                addMyNoteAction.Execute = delegate
                {
                    try
                    {
                        IMyNote myNote = _module.MyNoteList.CreateNew();
                        if (nodeList[0].Tag is IMyNoteGroup)
                        {
                            myNote.GroupId = ((IMyNoteGroup)nodeList[0].Tag).Id;
                        }
                        MyNoteDataView dataView = new MyNoteDataView();
                        dataView.Initialize(_shell);
                        dataView.MyNote = myNote;
                        _shell.DataViewControls.Add(dataView);
                    }
                    catch (Exception ex)
                    {
                        bool rethrow = DssEntlib.HandleUILayerException(ex);
                        if (rethrow)
                        {
                            throw;
                        }
                    }
                };

                actions.Add(addMyNoteAction);
            }

            // edit myNote
            if (myNoteNodes > 0)
            {
                NodeAction editMyNoteAction = new NodeAction(nodeList);
                editMyNoteAction.Text = Resources.MyNoteGroupExplorerPerspective_EditMyNote;
                editMyNoteAction.Enabled = myNoteNodes == 1 && groupNodes == 0 && branchNodes == 0;
                editMyNoteAction.ActionCode = ActionCodeEditMyNote;
                editMyNoteAction.ActionGroup = ActionGroupMyNote;
                editMyNoteAction.ShortcutKeys = Keys.Enter;
                editMyNoteAction.Execute = delegate
                {
                    try
                    {
                        MyNoteDataView dataView = null;
                        
                        // check if myNote is not already being edited
                        foreach (IDataViewControl dvControl in _shell.DataViewControls)
                        {
                            dataView = dvControl as MyNoteDataView;
                            if (dataView != null && dataView.MyNote == nodeList[0].Tag)
                            {
                                _shell.DataViewControls.Select(dataView);
                                break;
                            }
                            dataView = null;
                        }
                        
                        if (dataView == null)
                        {
                            dataView = new MyNoteDataView();
                            dataView.Initialize(_shell);
                            dataView.MyNote = nodeList[0].Tag as IMyNote;
                            _shell.DataViewControls.Add(dataView);
                        }
                    }
                    catch (Exception ex)
                    {
                        bool rethrow = DssEntlib.HandleUILayerException(ex);
                        if (rethrow)
                        {
                            throw;
                        }
                    }
                };

                actions.Add(editMyNoteAction);
            }

            return actions;
        }

        public override IList<ISelectable> GetSelectable(IList<ITreeNode> nodeList)
        {
            List<ISelectable> selectables = new List<ISelectable>();

            foreach (ITreeNode node in nodeList)
            {
                if (node.Tag is IMyNote)
                {
                    selectables.Add(_GetMyNoteSelectable((IMyNote)node.Tag));
                }
                else if (node.Tag is IMyNoteGroup)
                {
                    selectables.Add((IMyNoteGroup)node.Tag);
                }
            }

            return selectables;
        }

        private MyNoteProperties _GetMyNoteSelectable(IMyNote myNote)
        {
            if (!_myNotePropertiesCache.ContainsKey(myNote.Id))
            {
                _myNotePropertiesCache[myNote.Id] = new MyNoteProperties(myNote, _shell.Application);
            }

            return _myNotePropertiesCache[myNote.Id];
        }
    }
}
