﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using DHI.Solutions.Generic;
using MyNoteManager.Interfaces;

namespace MyNoteManager.UI
{
    internal class MyNoteProperties : ISelectable, INotifyPropertyChanged, IEntityWrapper
    {

        private IMyNote _myNote;
        private IApplication _app;
        private IMyNoteModule _myNoteModule;

        [PropertyName("MyNoteProperties_Name", typeof(Resources))]
        [PropertyDescription("MyNoteProperties_Name_Description", typeof(Resources))]
        [PropertyCategory("MyNoteProperties_MyNoteCategory", typeof(Resources))]
        public string Name
        {
            get
            {
                return _myNote.Name;
            }
            
            set
            {
                if (_myNote.Name != value)
                {
                    string oldName = _myNote.Name;
                    try
                    {
                        IChangeLogModule changeLogModule = _app.Modules.Get(typeof(IChangeLogModule)) as IChangeLogModule;
                        if (changeLogModule != null)
                        {
                            changeLogModule.AddChangeLogEntry(_myNote, "UI", "Changing name from " + _myNote.Name + " to " + value, null);
                        }
                        _myNote.Name = value;
                        _myNoteModule.MyNoteList.Update(_myNote);
                    }
                    catch (Exception ex)
                    {
                        Message.ShowUserInputErrorDialog(ex.Message, "Input Error");
                        _myNote.Name = oldName;
                    }
                }
            }
        }

        [PropertyName("MyNoteProperties_Text", typeof(Resources))]
        [PropertyDescription("MyNoteProperties_Text_Description", typeof(Resources))]
        [PropertyCategory("MyNoteProperties_MyNoteCategory", typeof(Resources))]
        public string Text
        {
            get
            {
                return _myNote.Text;
            }
        }

        public MyNoteProperties(IMyNote myNote, IApplication app)
        {
            _myNote = myNote;
            _app = app;
            _myNoteModule = app.Modules.Get("MyNote Manager") as IMyNoteModule;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public override string ToString()
        {
            return Resources.MyNote_EntityName + " - " + _myNote.Name;
        }

        private void _MyNote_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Name" ||
                e.PropertyName == "Text")
            {
                _OnPropertyChanged(e.PropertyName);
            }
        }

        private void _OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        [Browsable(false)]
        public IEntity Entity
        {
            get
            {
                return this._myNote;
            }
        }
    }
}
