﻿using System;
using System.ComponentModel;
using DHI.Solutions.Generic;
using System.Windows.Forms;
using DHI.Solutions.Generic.UI;
using MyNoteManager.Interfaces;
using DHI.Solutions.Generic.Interfaces;

namespace MyNoteManager.UI
{
    public class SaveButton : ToolStripButton, IToolStripButton
    {
        private BaseToolStrip _toolStrip;
        private MyNoteDataView _myNoteDataView;
        private IMyNote _myNote;

        public SaveButton(BaseToolStrip toolStrip, MyNoteDataView myNoteDataView)
        {
            _toolStrip = toolStrip;
            _myNoteDataView = myNoteDataView;
            _myNote = myNoteDataView.MyNote;

            _InitializeComponent();
            _UpdateButton();

            _myNote.PropertyChanged += new PropertyChangedEventHandler(_MyNote_PropertyChanged);
        }

        private void _InitializeComponent()
        {
            this.Name = this.GetType().Name;
            this.DisplayStyle = ToolStripItemDisplayStyle.Image;
            this.Image = Resources.Save.ToBitmap();
            this.Text = Resources.MyNoteToolStrip_SaveButton_Name;
            this.ToolTipText = Resources.MyNoteToolStrip_SaveButton_Name;
            this.Click += new EventHandler(_SaveButton_Click);
        }

        private void _MyNote_PropertyChanged(object sender, EventArgs e)
        {
            _UpdateButton();
        }

        private void _SaveButton_Click(object sender, EventArgs e)
        {
            try
            {
                _myNoteDataView.Save(); 
            }
            catch (Exception ex)
            {
                bool rethrow = DssEntlib.HandleRecoverableException(ex);
                if (rethrow)
                {
                    throw;
                }

                DHI.Solutions.Generic.Message.ShowWarningDialog(ex.Message, this.Text);
            }
            finally
            {
                _UpdateButton();
            }
        }

        private void _UpdateButton()
        {
            this.Enabled = _myNote.IsModified;
        }

        protected override void Dispose(bool disposing)
        {
            if (_myNote != null)
            {
                _myNote.PropertyChanged -= new PropertyChangedEventHandler(_MyNote_PropertyChanged);
            }
            
            this.Click -= new EventHandler(_SaveButton_Click);
            _toolStrip = null;
            _myNoteDataView = null;
            _myNote = null;

            base.Dispose(disposing);
        }
    }
}
