\connect :databasename

SET client_encoding = 'UTF8';
SET standard_conforming_strings = off;
SET check_function_bodies = false;
SET client_min_messages = warning;
SET escape_string_warning = off;

SET search_path = :workspace, pg_catalog;

CREATE OR REPLACE FUNCTION uuid_generate_v1()
RETURNS uuid
AS '$libdir/uuid-ossp', 'uuid_generate_v1'
VOLATILE STRICT LANGUAGE C;

CREATE OR REPLACE FUNCTION set_action(manager VARCHAR, actionkey VARCHAR) RETURNS integer AS $$
	DECLARE lead_role uuid;
	DECLARE member_role uuid;
	DECLARE reviewer_role uuid;
	BEGIN
		SELECT ID INTO lead_role FROM master.workspace_role WHERE "name"='workspace_lead';
		SELECT ID INTO member_role FROM master.workspace_role WHERE "name"='workspace_member';
		SELECT ID INTO reviewer_role FROM master.workspace_role WHERE "name"='workspace_reviewer';
	
		INSERT INTO action (id, manager_name, "name", workspace_role_id, enabled, visible) 
					select uuid_generate_v1(), manager, actionkey, lead_role, true, true;
		INSERT INTO action (id, manager_name, "name", workspace_role_id, enabled, visible) 
					select uuid_generate_v1(), manager, actionkey, member_role, true, true;
		INSERT INTO action (id, manager_name, "name", workspace_role_id, enabled, visible) 
					select uuid_generate_v1(), manager, actionkey, reviewer_role, false, true;

		RETURN 0;
	END;
	  $$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION alter_mynotemanager() RETURNS integer AS $$
	DECLARE myManager VARCHAR;
	BEGIN
		myManager ='MyNote Manager';

		DELETE FROM action WHERE manager_name = myManager;

		-- actions for CUT
		PERFORM set_action(myManager, 'ADD');
		PERFORM set_action(myManager, 'COPY');
		PERFORM set_action(myManager, 'PASTECOPY');
		PERFORM set_action(myManager, 'PASTEMOVE');
		PERFORM set_action(myManager, 'PASTELINK');
		PERFORM set_action(myManager, 'CUT');
		PERFORM set_action(myManager, 'DELETE');
		PERFORM set_action(myManager, 'RENAME');
		-- PERFORM set_action(myManager, 'REFRESH');
		-- PERFORM set_action(myManager, 'COPY_PATH');
		PERFORM set_action(myManager, 'DRAG');

		PERFORM set_action(myManager, 'ADD_ENTITY');
		PERFORM set_action(myManager, 'RENAME_ENTITY');

		PERFORM set_action(myManager, 'ADD_GROUP');
		PERFORM set_action(myManager, 'RENAME_GROUP');

		-- actions for EDITMYNOTE
		PERFORM set_action(myManager, 'EDITMYNOTE');

		RETURN 1;
	  END;
	  $$ LANGUAGE plpgsql;
	  
SELECT alter_mynotemanager();

DROP Function alter_mynotemanager();
DROP Function set_action(VARCHAR, VARCHAR);
DROP Function uuid_generate_v1();